function __httpRequest(method, url, postData, header, callback) {
    log("__httpRequest", url);
    var xhr = Titanium.Network.createHTTPClient();
    if (null !== header) for (var i in header) xhr.setRequestHeader(i, header[i]);
    xhr.open(method, url);
    xhr.onload = function() {
        log("HTTP LOAD ", this.responseText);
        var items = JSON.parse(this.responseText);
        callback.onSuccess(items);
    };
    xhr.onerror = function() {
        log("HTTP LOAD ", this.responseText);
        try {
            var items = JSON.parse(this.responseText);
            callback.onError && callback.onError(items.message);
        } catch (error) {
            callback.onError && callback.onError("error http request");
        }
    };
    Titanium.Network.online ? xhr.send(postData) : callback.onError && callback.onError();
}

function decodePoly(legGeometry) {
    log("decodePoly", legGeometry);
    try {
        return decodePolyUsingFloor(legGeometry, 1e5);
    } catch (err) {
        console.log("Exception trying to parse poly using 1E5 floorapix. Will try to return default 1/E6 floor. " + err);
        return decodePolyUsingFloor(legGeometry, 1e6);
    }
}

function decodePolyUsingFloor(legGeometry, floorApix) {
    var index = 0;
    var len = legGeometry.length;
    var lat = 0, lng = 0;
    var result_arr = [];
    while (len > index) {
        var b, shift = 0, result = 0;
        do {
            var c = legGeometry.charCodeAt(index++);
            b = c - 63;
            result |= (31 & b) << shift;
            shift += 5;
        } while (b >= 32);
        var dlat = 0 !== (1 & result) ? ~(result >> 1) : result >> 1;
        lat += dlat;
        shift = 0;
        result = 0;
        do {
            b = legGeometry.charCodeAt(index++) - 63;
            result |= (31 & b) << shift;
            shift += 5;
        } while (b >= 32);
        var dlng = 0 !== (1 & result) ? ~(result >> 1) : result >> 1;
        lng += dlng;
        var flon = lng / floorApix;
        var flat = lat / floorApix;
        if (-180 > flon || flon > 180 || -90 > flat || flat > 90) throw "Lat or lon are out of the permitted values, maybe decode algo was called with wrong params.";
        result_arr.push([ flon, flat ]);
    }
    return result_arr;
}

function log(tag, msg) {
    Ti.API.info("<<< " + tag + " >>> ");
    Ti.API.info(msg);
}

var CFG = Alloy.Globals.CONFIG;

var toolkit = require("eu.ilocate.toolkit");

exports.getPoiFile = function(fileName, callback) {
    __httpRequest("GET", CFG.url_portal_service + "?siteId=" + CFG.siteId + "&fileName=" + fileName, {}, "application/json", {
        onSuccess: function(data) {
            callback.success(JSON.stringify(data));
        },
        onError: function(err) {
            callback.error(err);
        }
    });
};

exports.getAccurateLocation = function(data, callback) {
    Ti.API.info(JSON.stringify(data));
    __httpRequest("POST", CFG.url_proxy, data, {}, callback);
};

exports.getIndoorLocation = function(data, callback) {
    var header = {
        "X-ILOCATE-USER-TOKEN": Alloy.Globals.TOKEN
    };
    __httpRequest("POST", CFG.url_proxy, JSON.stringify(data), header, callback);
};

exports.getGpsFromName = function(name, callback) {
    log("getGpsFromName", CFG.url_geocoder);
    __httpRequest("GET", CFG.url_geocoder + name, {}, null, callback);
};

exports.getRoutes = function(startGps, endGps, mode, callback) {
    function formatAMPM(date) {
        var hours = date.getHours();
        var minutes = date.getMinutes();
        var ampm = hours > 12 ? "pm" : "am";
        hours %= 12;
        hours = hours ? hours : 12;
        minutes = 10 > minutes ? "0" + minutes : minutes;
        var currentTime = hours + ":" + minutes + ampm;
        return currentTime;
    }
    var result = [];
    var date = new Date();
    var currentDate = (date.getUTCMonth() < 10 ? "0" + (date.getUTCMonth() + 1) : date.getUTCMonth() + 1) + "-" + (date.getUTCDate() < 10 ? "0" + date.getUTCDate() : date.getUTCDate()) + "-" + date.getUTCFullYear();
    var time = formatAMPM(new Date());
    var url = CFG.url_route + "plan?fromPlace=" + startGps + "&toPlace=" + endGps + "&time=" + time + "&date=" + currentDate + "&mode=" + mode + "&arriveBy=false&showIntermediateStops=false&_=1416308185460 ";
    log("URL", url);
    __httpRequest("GET", url, {}, "application/json", {
        onSuccess: function(data) {
            var itineraries = data["plan"]["itineraries"];
            for (var i = 0; i < itineraries.length; i++) {
                var container = {
                    legGeometry: [],
                    steps: []
                };
                var legs = itineraries[i]["legs"];
                for (var l = 0; l < legs.length; l++) {
                    container["legGeometry"] = decodePoly(legs[l]["legGeometry"]["points"]);
                    container["steps"] = legs[l]["steps"];
                }
                result.push(container);
            }
            callback.success(result);
        },
        onError: function(data) {
            log("getRoutes", JSON.stringify(data));
        }
    });
};

exports.registerUser = function(payload, callback) {
    var header = {
        "Content-Type": "application/json"
    };
    var url = CFG.url_openam + "/openam/json/users?_action=register";
    __httpRequest("POST", url, payload, header, callback);
};

exports.forgotPassword = function(payload, callback) {
    var header = {
        "Content-Type": "application/json"
    };
    var url = CFG.url_openam + "/openam/json/users?_action=forgotPassword";
    __httpRequest("POST", url, payload, header, callback);
};

exports.loginUser = function(username, password, callback) {
    var header = {
        "X-OpenAM-Username": username,
        "X-OpenAM-Password": password
    };
    __httpRequest("POST", CFG.url_openam + "/openam/json/authenticate", {}, header, callback);
};

exports.logout = function(callback) {
    var header = {
        iplanetDirectoryPro: Alloy.Globals.TOKEN,
        "content-type": "application/json"
    };
    var url = CFG.url_openam + "/openam/json/sessions/?_action=logout";
    __httpRequest("POST", url, {}, header, callback);
};